//******************************************************************************
//    File: tsxplug.h
//  Module: trueSpace eXtensions Installation Interface
//
//******************************************************************************

#ifndef TSXPLUG_H
#define TSXPLUG_H

//------------------------------------------------------------------------------
//	tsxData
//------------------------------------------------------------------------------

// Every trueSpace eXtension must define a function (see below) that returns
// information about the eXtension in the following `tsxData' structure.

struct tsxData
{
    char  m_Title[32];	//Brief name of this eXtension.
    char  m_Author[64]; //Name of author|company.
    short m_ResidBtnBmp;  //Resource id used for Button Bitmap
    short m_ResidBtnHelp; //Resource id used for Button Help Message
};

#ifndef __cplusplus
typedef struct tsxData tsxData;
#endif

// Each ts-eXtension will be associated with a button in the trueSpace
// interface. This requires:
//     - a bitmap (m_ResidBtnBmp), 34x34 pixels, 256 colors.
//     - and a brief help message (m_ResidBtnHelp) that gets
//       displayed on the status bar when a cursor moves over the button.


//------------------------------------------------------------------------------
//	TSX API Version Info
//------------------------------------------------------------------------------

// This is the version number for this copy of the API (SDK).
// To check the version number of an installed trueSpace, see "tsxMisc.h".

// Base alpha minor-version nbr
#define tsxVERSION_ALPHA_BASE -200
// Base beta minor-version nbr
#define tsxVERSION_BETA_BASE  -100

#define tsxVERSION_MAJOR    ((short) 320)
#define tsxVERSION_MINOR    ((short) 4)

//------------------------------------------------------------------------------
//	Access function names
//------------------------------------------------------------------------------

#define tsxGetData	tsxGetData
#define tsxOnLeftClick  tsxOnLeftClick
#define tsxOnRightClick tsxOnRightClick

#define FUNCNAME( x )  #x
#define TSX_GDFNAME FUNCNAME( tsxGetData )
#define TSX_LCFNAME FUNCNAME( tsxOnLeftClick )
#define TSX_DAFNAME FUNCNAME( tsxDeactivate )
#define TSX_RCFNAME FUNCNAME( tsxOnRightClick )

//------------------------------------------------------------------------------
//	Access function Return Values
//------------------------------------------------------------------------------

#define tsxPLUG_DONE       0
#define tsxPLUG_FAILED    -1
#define tsxPLUG_STAYON     1

//------------------------------------------------------------------------------
//	Access function declarations
//------------------------------------------------------------------------------

// Every ts-eXtension must define the following functions.

extern "C" {

// Function type for getting tsxData
typedef int (*tsxGetDataFP)(tsxData*, int);
// And the function ... copies data into supplied struct.
// `tsxid' is the id assigned to this extension while it is installed.
// You can use this function to initialize the eXtension, 
// and check version compatibility.
// Return: 
//  tsxPLUG_DONE = successful install and initialization of this eXtension.
//  tsxPLUG_FAILED = failed - do not install.
int tsxGetData( tsxData* tsx_data, int tsxid );

// Function type for Left-click on eXtension button.
// Invokes the eXtension.
typedef int (*tsxOnLeftClickFP)(void);
// And the function.
// Return:
//  tsxPLUG_STAYON = Keep button enabled, as eXtension stays active.
//  tsxPLUG_DONE = eXtension work completed, return to normal tS interface mode.
int tsxOnLeftClick(void);

// Function type for deactivating an active extension
typedef void (*tsxDeactivateFP)(void);
// And the function.
// This function called to deactivate an eXtension that requested staying
// activated thru `tsxOnLeftClick' returning `tsxPLUG_STAYON'.
void tsxDeactivate(void);

// Function type for Right-click on eXtension button.
// Invokes eXtension Parameter / About window.
typedef void (*tsxOnRightClickFP)(void);
// And the function.
void tsxOnRightClick(void);

}


//******************************************************************************
#endif // TSXPLUG_H
