//******************************************************************************
//    File: tsxErr.h
//  Module: trueSpace eXtensions API
//   Descr: TSX Error Codes
//******************************************************************************

#ifndef TSXERR_H
#define TSXERR_H

#include "tsxTypes.h"


//------------------------------------------------------------------------------
//	Error Codes returned by Functions with return type tsxERR
//------------------------------------------------------------------------------

#define tsxERR_SUCCESS 0

// Defined Error Codes

#define tsxERR_NOGOOD	   -1 // something generally went wrong
#define tsxERR_NO_MEMORY   -2 // no memory available
#define tsxERR_NULL_REF	   -3 // null pointer reference
#define tsxERR_BAD_INPUT   -4 // bad input data
#define tsxERR_FORMAT	   -5 // general bad data format
#define tsxERR_ABORT	   -8 // user abort request
#define tsxERR_REPORTED	  -10 // still an error but was reported below
#define tsxERR_UNIMPL	  -11 // feature unimplemented
#define tsxERR_IDENTIFIER -12 // bad identifier on data or record
#define tsxERR_VERSION 	  -13 // Correct file type but wrong version
#define tsxERR_NOT_FOUND  -14 // object not found
#define tsxERR_OVERFLOW	  -15 // data overflow for size of result
#define tsxERR_RANGE	  -16 // value out of range
#define tsxERR_INIT	  -18 // subsystem improperly initialized

// system errors
#define tsxERR_SYSERR_BASE -50
#define tsxERR_STDIO	  (tsxERR_SYSERR_BASE -0) // error occurred in io
#define tsxERR_MSWIN	  (tsxERR_SYSERR_BASE -1) // error occurred in windows

// If a function returns an error code not defined above, then treat it
// as an undefined error.


// A function may (rare) return positive success codes ...
inline tsxBOOL tsxSUCCEEDED( tsxERR err )
{
    return (tsxBOOL) (err >= 0);
}

inline tsxBOOL tsxFAILED( tsxERR err )
{
    return (tsxBOOL) (err < 0);
}


//*****************************************************************************
#endif // TSXERR_H
