//******************************************************************************
//    File: tsxExtOb.h
//  Module: trueSpace eXtensions API
//   Descr: Custom Object
//******************************************************************************

#ifndef TSXEXTOB_H
#define TSXEXTOB_H


#include "tsxTypes.h"


//------------------------------------------------------------------------------
//	Custom Object
//------------------------------------------------------------------------------

// Abstract type for file I/O.
struct TSXAPI tsxFILE{};

// Modes for FileIO.
enum tsxFILE_MODE { e_tsxFILE_BINARY = 0, e_tsxFILE_ASCII };

// FileIO.
enum tsxFILE_IO { e_tsxFILE_READ = 0, e_tsxFILE_WRITE };

// Changes of the object.
enum tsxOBJECT_CHANGE { e_tsxGEOMETRY = 1, e_tsxTOPOLOGY };

// FileIO.
enum tsxSELECT { e_tsxUNSELECTED = 0, e_tsxSELECTED };

// Flags for ApplyObmove.
#define tsxOMOVE_TRANSLATED	0x0001  /* a translation has been added */
#define tsxOMOVE_ROTATED	0x0002	/* a rotation has been added */
#define tsxOMOVE_SCALED		0x0004  /* a scaling has been added */

enum tsxCOLOR_MODE {

	e_tsxEDIT_CURPEN = 0,	/* draw whole node leaving pens in current state */
	e_tsxEDIT_ACTIVE,		/* draw whole node in active
							 * foreground color representation */
	e_tsxEDIT_ACTIVESUB,	/* draw whole node in active
							 * subobject foreground color representation */
	e_tsxEDIT_INACTIVE,		/* draw whole node in inactive background color
							 * representation (as partof larger tree) */
	e_tsxEDIT_INACTIVESUB,	/* draw whole node in inactive
							 * subobject color (as part of tree) background
							 * representation */
};


typedef tsxERR (*tsxObjectDeleteCallbackFP)( tsxSOBJ* pSobj );
		/* to free the object */

typedef tsxERR (*tsxObjectCloneSelfCallbackFP)( tsxSOBJ* pSobj, tsxSOBJ* pClone );
		/* make a duplicate of self */

typedef tsxERR (*tsxObjectFileIOCallbackFP)( tsxSOBJ* pSobj, tsxFILE *fd, tsxFILE_IO io, tsxFILE_MODE mode );
		/* io function */

typedef tsxERR (*tsxObjectRenderCallbackFP)( tsxSOBJ* pSobj, tsxCOLOR_MODE edit, CtsxRect* rect );
		/* draw whole node */
#define tsxERR_CLIPPED	0
#define tsxERR_VISIBLE	1

typedef tsxERR (*tsxObjectApplyObmoveCallbackFP)( tsxSOBJ* pSobj, CtsxTxmx3f* trans, unsigned short flags );
		/* apply transformation to object */

typedef tsxERR (*tsxObjectScanBCubeCallbackFP)( tsxSOBJ* pSobj, CtsxBBox3f* bc, CtsxTxmx3f* offset );
		/* return minmax for x,y,z for this object when offset by offset (optional) */

typedef tsxERR (*tsxObjectRayIntersectCallbackFP)( tsxSOBJ* pSobj, CtsxTxmx3f* torayspace );
		/* Test whether ray intersects object or not. Torayspace is a transform
		 * to put world space into rayspace where the ray is XYZ 0,0,1 
		 * Returns 0 if no hit 1 if hit, < 0 if error this is used for 
		 * object picking */
#define tsxERR_NOHIT	0
#define tsxERR_HIT		1

typedef tsxERR (*tsxObjectSelectCallbackFP)( tsxSOBJ* pSobj, tsxSELECT type );
		/* called when object is made or unmade current object */

typedef tsxERR (*tsxObjectHasChangedCallbackFP)( tsxSOBJ* pSobj, tsxOBJECT_CHANGE type );
		/* called when parent object is changed and other
		 * things need to know right away. */

#define tsxVERSION_OBJECT		2

typedef struct {
	int version;
	tsxObjectDeleteCallbackFP			extObjectDeleteCB;
	tsxObjectCloneSelfCallbackFP		extObjectCloneSelfCB;
	tsxObjectFileIOCallbackFP			extObjectFileIOCB;
	tsxObjectRenderCallbackFP			extObjectRenderCB;
	tsxObjectApplyObmoveCallbackFP		extObjectApplyObmoveCB;
	tsxObjectScanBCubeCallbackFP		extObjectScanBCubeCB;
	tsxObjectRayIntersectCallbackFP		extObjectRayIntersectCB;
	tsxObjectSelectCallbackFP			extObjectSelectCB;
	tsxObjectHasChangedCallbackFP		extObjectHasChangedCB;
} tsxObjectCallbackFP;

// Register an external type of object. Returns the type.
TSXAPIFN tsxSOBJTYPE tsxObjectDefine(
	int tsxid,							// eXtension id
	char* name,							// name of the object (max 16 characters)
	tsxObjectCallbackFP* objCallback,	// callback functions for the object
	long objDataLength					// length of data (may be 0L)
	);

// Create an object of the external type .
TSXAPIFN tsxSOBJ* tsxObjectCreate( tsxSOBJTYPE objType );

// Get an address of data in the object of the external type.
// Data of objDataLength length are a part of the object.
TSXAPIFN void* tsxObjectGetDataPtr( tsxSOBJ*pSobj );

// Get name of object.
TSXAPIFN const char* tsxObjectGetTypeName( tsxSOBJTYPE objType );


//------------------------------------------------------------------------------
//	Custom Data
//------------------------------------------------------------------------------

typedef tsxERR (*tsxDataDeleteCallbackFP)( void* pData );
		/* to free the data */

typedef tsxERR (*tsxDataCloneSelfCallbackFP)( void* pData, void* pClone );
		/* make a duplicate of self */

typedef tsxERR (*tsxDataFileIOCallbackFP)( void* pData, tsxFILE *fd, tsxFILE_IO io, tsxFILE_MODE mode );
		/* io function */

#define tsxVERSION_DATA		2

typedef struct {
	int version;
	tsxDataDeleteCallbackFP			extDataDeleteCB;
	tsxDataCloneSelfCallbackFP		extDataCloneSelfCB;
	tsxDataFileIOCallbackFP			extDataFileIOCB;
} tsxDataCallbackFP;

typedef int tsxDATATYPE;

// Register an external type of data. Returns the type.
TSXAPIFN tsxDATATYPE tsxDataDefine(
	int tsxid,							// eXtension id
	char* name,							// name of the data (max 16 characters)
	tsxDataCallbackFP* dataCallback,	// callback functions for the data
	long dataLength						// length of data
	);

// Add external data to the object.
TSXAPIFN void* tsxSobjCreateData( tsxSOBJ* pSobj, tsxDATATYPE dataType );

// Get external data of the object.
TSXAPIFN void* tsxSobjGetData( tsxSOBJ* pSobj, tsxDATATYPE dataType );

// Free external data from the object.
TSXAPIFN void tsxSobjDeleteData( tsxSOBJ* pSobj, tsxDATATYPE dataType );


// I/O functions
TSXAPIFN int tsxFILEerror(tsxFILE *fd);
TSXAPIFN int tsxFILEgetc(tsxFILE *fd);
TSXAPIFN char* tsxFILEgets(char *buf, int size, tsxFILE *fd);
TSXAPIFN int tsxFILEprintf(tsxFILE *fd, const char *fmt,...);
TSXAPIFN int tsxFILEputc(int ch, tsxFILE *fd);
TSXAPIFN int tsxFILEputs(const char *buf, tsxFILE *fd);
TSXAPIFN int tsxFILEread(void *buf, int size, int count, tsxFILE *fd);
TSXAPIFN int tsxFILEscanf(tsxFILE *fd, const char *fmt,...);
TSXAPIFN int tsxFILEseek(tsxFILE *fd, long offset, int where);
TSXAPIFN long tsxFILEtell(tsxFILE *fd);
TSXAPIFN int tsxFILEwrite(const void *buf, int size, int count, tsxFILE *fd);

//******************************************************************************
#endif // TSXEXTOB_H
