//******************************************************************************
//    File: tsxGroup.cpp
//  Module: trueSpace eXtensions API
//   Descr: Definition of tsxGROUP
//******************************************************************************

#ifndef TSXGROUP_H
#define TSXGROUP_H

#include "tsxTypes.h"

//------------------------------------------------------------------------------
//------------------------------------------------------------------------------

// Only tsxGNODE types can be grouped together. The children of a group
// may be any tsxGNODE derived type.

//------------------------------------------------------------------------------
//	Making Groups
//------------------------------------------------------------------------------

// The grouping mechanism (currently) only works with the currently selected obj.

// This is the same function as used in the [Glue as Child] button.
// (Currobj = currently selected object)
// Makes pGNode and Currobj children of a new Group obj, and
// the Group obj is placed where Currobj was in the Scene Graph.
// The new Group obj becomes the current selection, and acquires
// the previous Currobj's axes position and orientation.
// The animation timeline window is updated.
// pGNode and Currobj must have different Roots (`tsxGNodeGetRoot').
// Returns: 0 if no pGNode or Currobj, or not valid GNodes, or
//	      if pGNode is same as Currobj (no grouping done), else
//          ptr to the new Group obj.
TSXAPIFN tsxGROUP* tsxGroupNewWithCurrobj( tsxGNODE* pGNode );

// This is the same function as used in the [Glue as Sibling] button.
// (Currobj = currently selected object)
// IF Currobj is a Group node,
// THEN pGNode is added to it as another child;
// ELSE 
//   Makes pGNode and Currobj children of a new Group obj, and
//   the Group obj is placed where Currobj was in the Scene Graph.
//   The new Group obj becomes the current selection, and acquires
//   the previous Currobj's axes position and orientation.
// The animation timeline window is updated.
// pGNode and Currobj must have different Roots (`tsxGNodeGetRoot').
// pGNode must be a top level object (no parent groups above it).
// Returns: 0 if no pGNode or Currobj, or not valid GNodes, or
//	      if pGNode is same as Currobj (no grouping done), else
//          ptr to the (new or Currobj) Group obj.
TSXAPIFN tsxGROUP* tsxGroupAtCurrobj( tsxGNODE* pGNode );

// This is the same function as used in the [Unglue] button.
// Currobj is removed from its parent Group and added as a top level object
// to the scene.  Current selection is updated to next available sibling.
// Side Effect: If the Currobj has only one sibling before calling this
//    function, then the parent GROUP object is removed and the sibling takes
//    the place in the scene graph of the removed GROUP object. Thus this
//    function never results in a Group object with just one child.
// (Parent = parent GROUP of Currobj before calling this function)
// Returns:  
//   0   on failure (e.g. No Currobj, Currobj not part of group), else
//   ptr to oject that takes the place of Parent in the scene graph
//   (see note on Side Effect):
//	 = Parent if it had more than 2 children before this function, else
//       = the sibling that replaces Parent.
TSXAPIFN tsxGNODE* tsxGroupRemoveCurrobj();

//******************************************************************************
#endif //TSXGROUP_H
