//******************************************************************************
//    File: tsxMNode.h
//  Module: trueSpace eXtensions API
//   Descr: Definition of tsxMNODE
//******************************************************************************

#ifndef TSXMNODE_H
#define TSXMNODE_H


#include "tsxTypes.h"


//------------------------------------------------------------------------------
//------------------------------------------------------------------------------

// A tsxMNODE is the abstract type for Geometric Objects in a Scene.
// An MNode adds the following features to a GNode:
//     - Modification of Model Frame position and orientation,
//     - Visibility.
// All MNodes have a model frame that can be directly modified. TrueSpace
// visualy displays the model frame through the position and orientation of
// the MNode's Axes. TSXAPI allows direct modification of this model frame.


//------------------------------------------------------------------------------
//	MNode type check
//------------------------------------------------------------------------------

// e_tsxTRUE if pSobj is a MNode
TSXAPIFN tsxBOOL tsxIsMNode( tsxSOBJ* pSobj );


//------------------------------------------------------------------------------
//	Drawing/Rendering
//------------------------------------------------------------------------------

// Polyhdera are the only visible object types in a photo-rendered scene (other
// than lighting effects). Since Polyhdera may be members of Groups in a
// hierarchy, TSXAPI allows a Group MNode to be rendered, or set (in)visible.
// Only the Polyhdera members of the group are affected. Therefore, the following
// operations are meaningful for Group MNodes only if they have some Polyhdera
// as sub-objects.

// Returns e_tsxTRUE if pMNode is Visible in a rendered scene.
// A Group object may return TRUE even if it does not have any
// Polyhdera descendants.
TSXAPIFN tsxBOOL tsxMNodeIsVisible( tsxMNODE* pMNode );

// Makes pMNode Visible. Effect on next Draw operation.
TSXAPIFN void tsxMNodeSetVisible( tsxMNODE* pMNode );

// Makes pMNode Invisible. Effect on next Draw operation.
TSXAPIFN void tsxMNodeSetInvisible( tsxMNODE* pMNode );


//------------------------------------------------------------------------------
//	Model-frame Axes
//------------------------------------------------------------------------------

// The Axes define the object's own model frame (as different from the world
// frame). This frame is used for rigid body transformations
// (translation/rotation/scaling) (see tsxGNode).

// Returns e_tsxTRUE if MNode has its Axes displayed in tS.
TSXAPIFN tsxBOOL tsxMNodeAreAxesVisible( tsxMNODE* pMNode );

// Toggles (On/Off) the display of the current object's axes, if applicable.
TSXAPIFN void tsxCurrobjToggleAxes();

// Set the position of MNode's axes.
// Does not update views.
TSXAPIFN void tsxMNodeSetAxesPosition( tsxMNODE* pMNode, CtsxVector3f* posn );

// Set the orientation of MNode's axes.
// `axes' must point to valid (orthogonal, unit) axes.
// Does not update views.
TSXAPIFN void tsxMNodeSetAxesOrientation( tsxMNODE* pMNode, CtsxAxes3f* axes );


//******************************************************************************
#endif //TSXMNODE_H
