//******************************************************************************
//    File: tsxMisc.h
//  Module: trueSpace eXtensions API
//   Descr: Miscellaneous items
//******************************************************************************

#ifndef TSXMISC_H
#define TSXMISC_H

#include <windows.h>

#include "tsxTypes.h"

//------------------------------------------------------------------------------
//	Accessing an installed trueSpace's TSX Version Info
//------------------------------------------------------------------------------

// An eXtension may want to confirm that it is compatible with the version of 
// trueSpace it is being installed into (actually, compatible with the TSXAPI),
// by checking the version number of tsxapi.dll .
// 
// Note on TSX API Version numbering scheme:
// Major is the major release version of trueSpace, in hundreds.
// For example,
//   for trueSpace 3, Major = 300; 
//   for trueSpace 3.5 (should there be one) Major = 350.
// Minor is the minor release version of TSX API.
// Negative numbers in the minor refer to pre-release (alpha, beta) versions.
// 
// To check compatibility, 
// if a TSX requires any version on or after [baseMajor, baseMinor],
// check for:  (Major >= baseMajor) && 
//             (Minor >= baseMinor)

// Accessing an installed trueSpace's TSX Version Info.  Values placed in
// the arguments.
TSXAPIFN void tsxGetTsVersionNbrs(
    short* pMajor,	// Major version nbr
    short* pMinor	// Minor version nbr
    );


//------------------------------------------------------------------------------
//	Accessing The primary view-window client area
//------------------------------------------------------------------------------

// Returns handle to the main view window.
TSXAPIFN HWND tsxGetMainViewHwnd(void);


//------------------------------------------------------------------------------
//	Requesting (De)Activation
//------------------------------------------------------------------------------

// Requests a left-click action on the extension's button (see tsxOnLeftClick).
// Useful when, e.g.,  an extension wants to leave a panel on the screen even
// when deactivated, and then requests re-activation when a button is clicked
// on this panel. Also useful to deactivate an active extension when user
// exits from extensions parameter panel.
TSXAPIFN tsxERR tsxForceLeftClick(
    int tsxid // extension-id (see tsxGetData)
    );


//------------------------------------------------------------------------------
//	Heap Memory
//------------------------------------------------------------------------------

// Use these memory routines when allocating memory for trueSpace,
// e.g. vertex-array for Polyhedra (tsxPolyh.h).

TSXAPIFN void* tsxMalloc( unsigned int size );
TSXAPIFN void* tsxCalloc( unsigned int num, unsigned int size );
// You MUST use the following two functions to reallocate or free
// memory blocks allocated by the above functions.
TSXAPIFN void* tsxRealloc( void* memblock, unsigned int size );
TSXAPIFN void tsxFree( void* memblock );

//******************************************************************************
#endif // TSXMISC_H
