//******************************************************************************
//    File: tsxScene.cpp
//  Module: trueSpace eXtensions API
//   Descr: Definition of tsxSCENE
//******************************************************************************

#ifndef TSXSCENE_H
#define TSXSCENE_H

#include "tsxTypes.h"

//------------------------------------------------------------------------------
//------------------------------------------------------------------------------

// There is always one and only one scene in trueSpace.
// tsxSCENE adds the following information to tsxSOBJ:
//      - Fog parms


//------------------------------------------------------------------------------
//	Scene, Graph access
//------------------------------------------------------------------------------

// Get the trueSpace scene.
TSXAPIFN tsxSCENE* tsxGetScene();

// Adds pGNode as top-level member of scene, and makes it the current selection.
// IF bNoDraw is TRUE THEN the object will not be drawn in this function call.
// pGNode must be a new object not already in Scene.
TSXAPIFN void tsxSceneAddObject( tsxGNODE* pGNode, tsxBOOL bNoDraw );

// Get the first top level GNode.
// The rest can be accessed using `tsxGNodeGetNext'.
TSXAPIFN tsxGNODE* tsxSceneGetFirstNode();

//------------------------------------------------------------------------------
//	Fog
//------------------------------------------------------------------------------

// Returns tsxTRUE for Fog On.
TSXAPIFN tsxBOOL tsxSceneIsFogOn();

// Use tsxTRUE to set Fog On.
TSXAPIFN void tsxSceneSwitchFog( tsxBOOL fog_on );

//
TSXAPIFN void tsxSceneGetFogParms(
    unsigned int* pFogRed,	//0 - 255
    unsigned int* pFogBlue,	//0 - 255
    unsigned int* pFogGreen,	//0 - 255
    int* 	  pFogNear,
    int* 	  pFogFar,
    int*	  pFogMax
    );

//
TSXAPIFN void tsxSceneSetFogParms(
    unsigned int FogRed,	//0 - 255
    unsigned int FogBlue,	//0 - 255
    unsigned int FogGreen,	//0 - 255
    int 	 FogNear,
    int 	 FogFar,
    int	  	 FogMax
    );



//------------------------------------------------------------------------------
//	Rendering
//------------------------------------------------------------------------------

// (Re)Draw the scene in the active View, in the View's rendering mode.
TSXAPIFN void tsxSceneDraw();


//******************************************************************************
#endif //TSXSCENE_H
